package org.jboss.jms.client;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TransactionRolledBackException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class TestConsumer {

    int maxRetries = 30;
    long receiveTimeOut = 3000;
    int counter = 0;
    int ackAfter = 10;
    String hostname = "localhost";
    String queueJndiName = "";
    
    public static void main(String[] args) throws Exception {

        if (args.length != 2) {
            System.out.println("Set parameters \"hostname\" and \"jndi_name_of_the_queue\"");
            System.exit(1);
        }
        TestConsumer consumer = new TestConsumer(args[0], args[1]);
//        TestConsumer consumer = new TestConsumer("messaging-18", "jms/queue/testQueue0");
        consumer.startClient();
    }


    public TestConsumer(String hostname, String queueJndiName)  {
        this.hostname = hostname;
        this.queueJndiName = queueJndiName;
    }

    void startClient() throws Exception  {

        Map<String, String> idChecked = new HashMap<String, String>();
        Context context = null;
        Connection con = null;
        Session session = null;

        try {

            final Properties env = new Properties();
            env.put(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.naming.remote.client.InitialContextFactory");
            env.put(Context.PROVIDER_URL, "remote://" + hostname + ":4447");

            context = new InitialContext(env);

            ConnectionFactory cf = (ConnectionFactory) (ConnectionFactory) context.lookup("jms/RemoteConnectionFactory");
            con = (Connection) cf.createConnection();

            con.start();

            session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);
            Queue queue = (Queue) context.lookup(queueJndiName);

            MessageConsumer consumer = session.createConsumer(queue);
            
            Message message = null;
            Message lastMessage = null;
            
            while ((message = receiveMessage(consumer)) != null) {

                counter++;
                
                if (counter % ackAfter != 0) {
                        
                    acknowledgeMessage(message);
                    System.out.println(counter + ". got message id = " + message.getJMSMessageID() + " message - ACKED");                        
                } else {
                    System.out.println(counter + ". got message id = " + message.getJMSMessageID() + " message counter: "
                        + message.getStringProperty("count"));
                }
                Thread.sleep(10);
                lastMessage = message;
            }

            if (lastMessage != null)    {
                acknowledgeMessage(message);
                lastMessage.acknowledge();
            }
            System.out.println("Receiver ended - number of messages " + counter);

        } finally {
            if (session != null) {
                session.close();
            }
            if (con != null) {
                try {
                    con.close();
                } catch (JMSException e) {
                    e.printStackTrace();
                }
            }
            if (context != null) {
                try {
                    context.close();
                } catch (NamingException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    public Message receiveMessage(MessageConsumer consumer) throws Exception {

        Message msg;
        int numberOfRetries = 0;

        // receive message with retry
        while (numberOfRetries < maxRetries) {

            try {

                msg = consumer.receive(receiveTimeOut);
                return msg;

            } catch (JMSException ex) {
                numberOfRetries++;
                System.out.println("RETRY receive for host: " + hostname + ", Trying to receive message with count: " + (counter));
            }
        }

        throw new Exception("FAILURE - MaxRetry reached for receiver for node: " + hostname);
    }

    public void acknowledgeMessage(Message message) throws Exception {

        int numberOfRetries = 0;


        while (numberOfRetries < maxRetries) {
            try {

                message.acknowledge();

                return;

            } catch (TransactionRolledBackException ex) {
                System.out.println("TransactionRolledBackException thrown during acknowledge. Receiver for node: " + hostname + ". Received message - count: "
                        + counter + ", messageId:" + message.getJMSMessageID());
                // all unacknowledge messages will be received again
                counter = counter - ackAfter;
                ex.printStackTrace();                

            } catch (JMSException ex) {
                // now it's screwed because we don't have response for sent ACK
                // next receive can have duplicates or new messages
               
                System.out.println("JMSException thrown during acknowledge. Receiver for node: " + hostname + ". Received message - count: "
                        + counter + ", messageId:" + message.getJMSMessageID());                        

                ex.printStackTrace();
                numberOfRetries++;
            }
        }
    }
}
