package org.jboss.jms.client;

import java.util.Properties;
import java.util.UUID;
import javax.jms.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class TestProducer {

    private int numberOfRetries = 0;
    private static int maxRetries = 50;
    int counter = 1;

    private Message createMessage(Session session, int counter) throws Exception {
//        TextMessage message = session.createTextMessage(new String(new byte[1024]));
//        message.setIntProperty("count", counter);
        ClientMixMessageBuilder builder = null;
        if (builder == null) {
            builder = new ClientMixMessageBuilder(10, 100);
        }
        builder.setCounter(counter);

        return builder.createMessage(session);
    }

    public static void main(String[] args) {
        if (args.length != 3) {
            System.out.println("Set parameters \"hostname\" and \"jndi_name_of_the_queue\" ant \"number_of_messages\"");
            System.exit(1);
        }
        TestProducer testProducer = new TestProducer();
        testProducer.startClient(args);
    }

    public void startClient(String[] args) {

        final int MESSAGES_COUNT = Integer.valueOf(args[2]);

        Context context = null;
        Connection con = null;
        Session session = null;

        try {
            System.out.println("remote://" + args[0] + ":4447");
            final Properties env = new Properties();
            env.put(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.naming.remote.client.InitialContextFactory");
            env.put(Context.PROVIDER_URL, "remote://" + args[0] + ":4447");

            context = new InitialContext(env);

            ConnectionFactory cf = (ConnectionFactory) context.lookup("jms/RemoteConnectionFactory");

            Queue queue = (Queue) context.lookup(args[1]);

            con = cf.createConnection();

            session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);

            MessageProducer producer = session.createProducer(queue);

            Message msg = null;

            while (counter < MESSAGES_COUNT) {

                msg = createMessage(session, counter);

                msg.setStringProperty("_HQ_DUPL_ID", (UUID.randomUUID().toString() + System.currentTimeMillis()));

                sendMessage(producer, msg);

                System.out.println("Sent message with property count: " + counter + ", messageId:" + msg.getJMSMessageID());


                counter++;
            }

            producer.close();

        } catch (Exception e) {

            e.printStackTrace();

        } finally {

            if (session != null) {
                try {
                    session.close();
                } catch (JMSException e) {
                    e.printStackTrace();
                }
            }
            if (con != null) {
                try {
                    con.close();
                } catch (JMSException e) {
                    e.printStackTrace();
                }
            }
            if (context != null) {
                try {
                    context.close();
                } catch (NamingException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private void sendMessage(MessageProducer producer, Message msg) throws Exception {

        int numberOfRetries = 0;

        while (numberOfRetries < maxRetries) {

            try {

                producer.send(msg);

                counter++;

                return;

            } catch (JMSException ex) {

                try {
                    System.out.println("SEND RETRY - Sent message with property count: " + counter + ", messageId:" + msg.getJMSMessageID() + ((msg.getStringProperty("_HQ_DUPL_ID") != null) ? ", _HQ_DUPL_ID=" + msg.getStringProperty("_HQ_DUPL_ID") : ""));
                } catch (JMSException e) {
                } // ignore

                numberOfRetries++;
            }
        }

        // this is an error - here we should never be because max retrie expired
        throw new Exception("FAILURE - MaxRetry reached for producer during sent message with property count: " + counter + ", messageId:" + msg.getJMSMessageID());

    }
}
