package org.jboss.jms.client;

import java.util.Properties;
import java.util.UUID;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.*;

public class TestProducerSimple {

    private int numberOfRetries = 0;
    private static int maxRetries = 50;

    private Message createMessage(Session session, int counter) throws Exception {
        TextMessage message = session.createTextMessage(new String(new byte[1024]));
        
        message.setIntProperty("count", counter);

        return message;
    }

    public static void main(String[] args) {

//        if (args.length != 3) {
//            System.out.println("Set parameters \"hostname\" and \"jndi_name_of_the_queue\" and \"number_of_messages\"");
//            System.exit(1);
//        }
        TestProducerSimple testProducer = new TestProducerSimple();
        testProducer.startClient();
    }

    public void startClient() {

        final int MESSAGES_COUNT = 1000;

        Context context = null;
        Connection con = null;
        Session session = null;

        try {

            final Properties env = new Properties();
            env.put(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.naming.remote.client.InitialContextFactory");
            env.put(Context.PROVIDER_URL, "remote://messaging-19:4447");
//            env.put(Context.SECURITY_PRINCIPAL, "guest");
//            env.put(Context.SECURITY_CREDENTIALS, "g");
            
            context = new InitialContext(env);

            ConnectionFactory cf = (ConnectionFactory) context.lookup("jms/RemoteConnectionFactory");

            Queue queue = (Queue) context.lookup("jms/queue/testQueue0");

            con = cf.createConnection();

            session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);

            MessageProducer producer = session.createProducer(queue);

            int i = 1;

            Message msg = null;

            while (i <= MESSAGES_COUNT) {

                msg = createMessage(session, i);

                try {

                    producer.send(msg);

                    System.out.println("Sent message with property count: " + i + ", messageId:" + msg.getJMSMessageID());


                } catch (Exception ex) {
                    // exception can be thrown when something happens with server
                    resendMessage(producer, msg, i);
                    ex.printStackTrace();
                }

                i++;
            }

            producer.close();

        } catch (Exception e) {

            e.printStackTrace();

        } finally {

            if (session != null) {
                try {
                    session.close();
                } catch (JMSException e) {
                    e.printStackTrace();
                }
            }
            if (con != null) {
                try {
                    con.close();
                } catch (JMSException e) {
                    e.printStackTrace();
                }
            }
            if (context != null) {
                try {
                    context.close();
                } catch (NamingException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private void resendMessage(MessageProducer producer, Message msg, int i) {
        try {
            if (numberOfRetries > maxRetries) {
                try {
                    throw new Exception("Number of retries (" + numberOfRetries + ") is greater than limit (" + maxRetries + ").");
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
            Thread.sleep(3000);
            numberOfRetries++;
            System.out.println("Sender RETRY (" + numberOfRetries + ") Sent message: " + i);
            producer.send(msg);
        } catch (JMSException ex) {
            ex.printStackTrace();
            resendMessage(producer, msg, i);
        } catch (InterruptedException ex) {
            ex.printStackTrace();
        }

    }
}
