package org.jboss.jms.client;

import javax.jms.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.UUID;

public class TestProducerTransacted {


    private static int maxRetries = 50;

    int count = 1;
    MessageProducer producer;

    private List<Message> listOfSentMessages = new ArrayList<Message>();
    private List<Message> listOfMessagesToBeCommited = new ArrayList<Message>();


    private Message createMessage(Session session, int counter) throws Exception {
//        TextMessage message = session.createTextMessage(new String(new byte[1024 * 1024]));
        TextMessage  message = session.createTextMessage();
        message.setText(new String(new char[10 * 1024 * 1024]));
        message.setStringProperty("_HQ_DUPL_ID", String.valueOf(UUID.randomUUID()) + counter);
        message.setIntProperty("count", counter);

        return message;
    }

    public static void main(String[] args) {
//        args = new String[3];
//        args[0] = "127.0.0.1";
//        args[1] = "jms/queue/testQueue0";
//        args[2] = "20";
        TestProducerTransacted testProducer = new TestProducerTransacted();
        testProducer.startClient(args);
    }

    public void startClient(String[] args) {

        final int MESSAGES_COUNT = Integer.valueOf(args[2]);

        Context context = null;
        Connection con = null;
        Session session = null;

        try {

            final Properties env = new Properties();
            env.put(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.naming.remote.client.InitialContextFactory");
            env.put(Context.PROVIDER_URL, "remote://" + args[0] + ":4447");

            context = new InitialContext(env);

            ConnectionFactory cf = (ConnectionFactory) context.lookup("jms/RemoteConnectionFactory");

            Queue queue = (Queue) context.lookup(args[1]);

            con = cf.createConnection();

            session = con.createSession(true, Session.SESSION_TRANSACTED);

            producer = session.createProducer(queue);

            Message msg = null;

            while (count <= MESSAGES_COUNT) {

                msg = createMessage(session, count);

                sendMessage(msg);

                listOfMessagesToBeCommited.add(msg);

                if (count % 2 == 0) {

                    commitSession(session);
                    listOfSentMessages.addAll(listOfMessagesToBeCommited);
                    System.out.println("COMMIT - session was commited. Last message with property count: " + count
                            + ", messageId:" + msg.getJMSMessageID());
                    listOfMessagesToBeCommited.clear();

                }
            }

            commitSession(session);

            producer.close();

        } catch (Exception e) {

            e.printStackTrace();

        } finally {

            if (session != null) {
                try {
                    session.close();
                } catch (JMSException e) {
                    e.printStackTrace();
                }
            }
            if (con != null) {
                try {
                    con.close();
                } catch (JMSException e) {
                    e.printStackTrace();
                }
            }
            if (context != null) {
                try {
                    context.close();
                } catch (NamingException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private void sendMessage(Message msg) {
        int numberOfRetries = 0;
        try {
            if (numberOfRetries > maxRetries) {
                try {
                    throw new Exception("Number of retries (" + numberOfRetries + ") is greater than limit (" + maxRetries + ").");
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }

            if (numberOfRetries > 0) {
                System.out.println("Retry sent - number of retries: (" + numberOfRetries + ") message: " + msg.getJMSMessageID() + ", count: " + count);
            }
            numberOfRetries++;
            producer.send(msg);
            System.out.println("Sent message with property count: " + count + ", messageId:" + msg.getJMSMessageID()
                    + " dupId: " + msg.getStringProperty("_HQ_DUPL_ID"));
            count++;
        } catch (JMSException ex) {
            ex.printStackTrace();
            sendMessage(msg);
        }
    }

    private void commitSession(Session session) throws Exception {
        int numberOfRetries = 0;
        while (true) {
            try {

                // try commit
                session.commit();

                // if successful -> return
                return;

            } catch (TransactionRolledBackException ex) {
                // if transaction rollback exception -> send messages again and commit
                ex.printStackTrace();

                // don't repeat this more than once, this can't happen
                if (numberOfRetries > 0) {
                    throw new Exception("Fatal error. TransactionRolledBackException was thrown more than once for one commit. Message count: " + count
                            + " Client will terminate.");
                }

                count -= listOfMessagesToBeCommited.size();

                for (Message m : listOfMessagesToBeCommited) {
                    sendMessage(m);
                }

                numberOfRetries++;

            } catch (JMSException ex) {
                // if jms exception -> send messages again and commit (in this case server will throw away possible duplicates because dup_id  is set so it's safe)
                ex.printStackTrace();

                // don't repeat this more than once - it's exception because of duplicates
                if (numberOfRetries > 0) {
                    return;
                }

                count -= listOfMessagesToBeCommited.size();

                for (Message m : listOfMessagesToBeCommited) {
                    sendMessage(m);
                }
                numberOfRetries++;
            }
        }
    }

    /**
     * @return the listOfMessagesToBeCommited
     */
    public List<Message> getListOfMessagesToBeCommited() {
        return listOfMessagesToBeCommited;
    }

    /**
     * @param listOfMessagesToBeCommited the listOfMessagesToBeCommited to set
     */
    public void setListOfMessagesToBeCommited(List<Message> listOfMessagesToBeCommited) {
        this.listOfMessagesToBeCommited = listOfMessagesToBeCommited;
    }

    /**
     * @return the listOfSentMessages
     */
    public List<Message> getListOfSentMessages() {
        return listOfSentMessages;
    }

    /**
     * @param listOfSentMessages the listOfSentMessages to set
     */
    public void setListOfSentMessages(List<Message> listOfSentMessages) {
        this.listOfSentMessages = listOfSentMessages;
    }

}
